import {
  DefaultLogger
} from "./chunk-VAS3X5GJ.js";
import {
  isString
} from "./chunk-EOHXRMVQ.js";
import {
  isPromise
} from "./chunk-FSDCRQB3.js";
import {
  arrayFilterInPlace
} from "./chunk-AEBHR4XF.js";

// src/common/dispose-defer.ts
function callDisposer(disposable) {
  let result;
  if (typeof disposable === "function")
    result = disposable();
  else if (isPromise(disposable))
    result = disposable;
  else if (typeof disposable.dispose === "function")
    result = disposable.dispose();
  else if (isPromise(disposable.dispose))
    result = disposable.dispose;
  if (isPromise(result))
    return result;
}
function useDispose(config) {
  let opt = config;
  if (opt != null) {
    if (isString(opt))
      opt = { name: opt };
    else if ("debug" in opt && "label" in opt)
      opt = { name: opt.label, log: opt };
  }
  const name = opt?.name;
  const log = opt?.log ?? DefaultLogger("zeed:dispose");
  let disposed = 0;
  const tracked = [];
  function untrack(disposable) {
    if (disposable != null && tracked.includes(disposable)) {
      arrayFilterInPlace(tracked, (el) => el !== disposable);
      const result = callDisposer(disposable);
      if (isPromise(result))
        return result;
    }
  }
  function track(obj) {
    if (obj == null)
      return;
    tracked.unshift(obj);
    return () => untrack(obj);
  }
  function dispose(strictSync = false) {
    if (name)
      log.debug(`dispose "${name}": ${tracked.length} entries`);
    disposed += 1;
    const promises = [];
    while (tracked.length > 0) {
      const fn = tracked[0];
      const result = untrack(fn);
      if (isPromise(result)) {
        if (strictSync)
          throw new Error(`Async disposable found: ${fn} -> ${result}`);
        else
          promises.push(result);
      }
    }
    if (promises.length > 0)
      return Promise.all(promises);
  }
  function disposeSync() {
    void dispose(true);
  }
  return Object.assign(dispose, {
    /** Counter that incremends, each time dispose has been called */
    disposed,
    add: track,
    remove: untrack,
    /** @deprecated use add */
    track,
    /** @deprecated use remove */
    untrack,
    dispose,
    disposeSync,
    sync: disposeSync,
    /** @deprecated use dispose */
    exec: dispose,
    getSize() {
      return tracked.length;
    },
    isDisposed() {
      return tracked.length <= 0;
    }
  });
}
function useDefer(config = {}) {
  const { mode = "fifo" } = config;
  const steps = [];
  const exec = async (expectSync = false) => {
    while (steps.length > 0) {
      const step = steps[0];
      arrayFilterInPlace(steps, (el) => el !== step);
      if (typeof step === "function") {
        const result = step();
        if (isPromise(result)) {
          if (expectSync) {
            throw new Error(
              `Expected sync only function, but found async: ${step}`
            );
          }
          await result;
        }
      } else if (isPromise(step)) {
        if (expectSync) {
          throw new Error(
            `Expected sync only function, but found async: ${step}`
          );
        }
        await step;
      } else {
        throw new Error(`Unhandled disposable: ${step}`);
      }
    }
  };
  const add = (obj) => {
    if (mode === "lifo")
      steps.unshift(obj);
    else
      steps.push(obj);
  };
  return Object.assign(exec, {
    add,
    exec,
    getSize() {
      return steps.length;
    }
  });
}

export {
  useDispose,
  useDefer
};
//# sourceMappingURL=chunk-KGD7JR27.js.map