import {
  createUint8ArrayViewFromArrayBuffer
} from "./chunk-MWUFUBY5.js";
import {
  encodeUtf8,
  getUtf8TextEncoder
} from "./chunk-GKKLBSFA.js";
import {
  BIT7,
  BIT8,
  BITS31,
  BITS6,
  BITS7,
  BITS8
} from "./chunk-RNL46FQ3.js";

// src/common/bin/lib0/encoding.ts
var Encoder = class {
  constructor() {
    this.cpos = 0;
    this.cbuf = new Uint8Array(100);
    this.bufs = [];
  }
};
function createEncoder() {
  return new Encoder();
}
function length(encoder) {
  let len = encoder.cpos;
  for (let i = 0; i < encoder.bufs.length; i++)
    len += encoder.bufs[i].length;
  return len;
}
function encodeToUint8Array(encoder) {
  const uint8arr = new Uint8Array(length(encoder));
  let curPos = 0;
  for (let i = 0; i < encoder.bufs.length; i++) {
    const d = encoder.bufs[i];
    uint8arr.set(d, curPos);
    curPos += d.length;
  }
  uint8arr.set(createUint8ArrayViewFromArrayBuffer(encoder.cbuf.buffer, 0, encoder.cpos), curPos);
  return uint8arr;
}
function verifyLen(encoder, len) {
  const bufferLen = encoder.cbuf.length;
  if (bufferLen - encoder.cpos < len) {
    encoder.bufs.push(createUint8ArrayViewFromArrayBuffer(encoder.cbuf.buffer, 0, encoder.cpos));
    encoder.cbuf = new Uint8Array(Math.max(bufferLen, len) * 2);
    encoder.cpos = 0;
  }
}
function write(encoder, num) {
  const bufferLen = encoder.cbuf.length;
  if (encoder.cpos === bufferLen) {
    encoder.bufs.push(encoder.cbuf);
    encoder.cbuf = new Uint8Array(bufferLen * 2);
    encoder.cpos = 0;
  }
  encoder.cbuf[encoder.cpos++] = num;
}
function set(encoder, pos, num) {
  let buffer = null;
  for (let i = 0; i < encoder.bufs.length && buffer === null; i++) {
    const b = encoder.bufs[i];
    if (pos < b.length)
      buffer = b;
    else
      pos -= b.length;
  }
  if (buffer === null) {
    buffer = encoder.cbuf;
  }
  buffer[pos] = num;
}
var writeUint8 = write;
var setUint8 = set;
function writeUint16(encoder, num) {
  write(encoder, num & BITS8);
  write(encoder, num >>> 8 & BITS8);
}
function setUint16(encoder, pos, num) {
  set(encoder, pos, num & BITS8);
  set(encoder, pos + 1, num >>> 8 & BITS8);
}
function writeUint32(encoder, num) {
  for (let i = 0; i < 4; i++) {
    write(encoder, num & BITS8);
    num >>>= 8;
  }
}
function writeUint32BigEndian(encoder, num) {
  for (let i = 3; i >= 0; i--)
    write(encoder, num >>> 8 * i & BITS8);
}
function setUint32(encoder, pos, num) {
  for (let i = 0; i < 4; i++) {
    set(encoder, pos + i, num & BITS8);
    num >>>= 8;
  }
}
function writeVarUint(encoder, num) {
  while (num > BITS7) {
    write(encoder, BIT8 | BITS7 & num);
    num = Math.floor(num / 128);
  }
  write(encoder, BITS7 & num);
}
function isNegativeZero(n) {
  return n !== 0 ? n < 0 : 1 / n < 0;
}
function writeVarInt(encoder, num) {
  const isNegative = isNegativeZero(num);
  if (isNegative)
    num = -num;
  write(encoder, (num > BITS6 ? BIT8 : 0) | (isNegative ? BIT7 : 0) | BITS6 & num);
  num = Math.floor(num / 64);
  while (num > 0) {
    write(encoder, (num > BITS7 ? BIT8 : 0) | BITS7 & num);
    num = Math.floor(num / 128);
  }
}
function writeUint8Array(encoder, uint8Array) {
  const bufferLen = encoder.cbuf.length;
  const cpos = encoder.cpos;
  const leftCopyLen = Math.min(bufferLen - cpos, uint8Array.length);
  const rightCopyLen = uint8Array.length - leftCopyLen;
  encoder.cbuf.set(uint8Array.subarray(0, leftCopyLen), cpos);
  encoder.cpos += leftCopyLen;
  if (rightCopyLen > 0) {
    encoder.bufs.push(encoder.cbuf);
    encoder.cbuf = new Uint8Array(Math.max(bufferLen * 2, rightCopyLen));
    encoder.cbuf.set(uint8Array.subarray(leftCopyLen));
    encoder.cpos = rightCopyLen;
  }
}
function writeVarUint8Array(encoder, uint8Array) {
  writeVarUint(encoder, uint8Array.byteLength);
  writeUint8Array(encoder, uint8Array);
}
var _strBuffer;
var _maxStrBSize;
function _writeVarStringNative(encoder, str) {
  if (_strBuffer == null) {
    _strBuffer = new Uint8Array(3e4);
    _maxStrBSize = _strBuffer.length / 3;
  }
  if (str.length < _maxStrBSize) {
    const written = getUtf8TextEncoder().encodeInto(str, _strBuffer).written || 0;
    writeVarUint(encoder, written);
    for (let i = 0; i < written; i++)
      write(encoder, _strBuffer[i]);
  } else {
    writeVarUint8Array(encoder, encodeUtf8(str));
  }
}
function _writeVarStringPolyfill(encoder, str) {
  const encodedString = unescape(encodeURIComponent(str));
  const len = encodedString.length;
  writeVarUint(encoder, len);
  for (let i = 0; i < len; i++)
    write(encoder, encodedString.codePointAt(i));
}
function writeVarString(encoder, str) {
  return getUtf8TextEncoder()?.encodeInto ? _writeVarStringNative(encoder, str) : _writeVarStringPolyfill(encoder, str);
}
function writeBinaryEncoder(encoder, append) {
  return writeUint8Array(encoder, encodeToUint8Array(append));
}
function writeOnDataView(encoder, len) {
  verifyLen(encoder, len);
  const dview = new DataView(encoder.cbuf.buffer, encoder.cpos, len);
  encoder.cpos += len;
  return dview;
}
function writeFloat32(encoder, num) {
  return writeOnDataView(encoder, 4).setFloat32(0, num, false);
}
function writeFloat64(encoder, num) {
  return writeOnDataView(encoder, 8).setFloat64(0, num, false);
}
function writeBigInt64(encoder, num) {
  return writeOnDataView(encoder, 8).setBigInt64(0, num, false);
}
function writeBigUint64(encoder, num) {
  return writeOnDataView(encoder, 8).setBigUint64(0, num, false);
}
var floatTestBed;
function isFloat32(num) {
  if (floatTestBed == null)
    floatTestBed = new DataView(new ArrayBuffer(4));
  floatTestBed.setFloat32(0, num);
  return floatTestBed.getFloat32(0) === num;
}
function writeAny(encoder, data) {
  switch (typeof data) {
    case "string":
      write(encoder, 119);
      writeVarString(encoder, data);
      break;
    case "number":
      if (Number.isInteger(data) && Math.abs(data) <= BITS31) {
        write(encoder, 125);
        writeVarInt(encoder, data);
      } else if (isFloat32(data)) {
        write(encoder, 124);
        writeFloat32(encoder, data);
      } else {
        write(encoder, 123);
        writeFloat64(encoder, data);
      }
      break;
    case "bigint":
      write(encoder, 122);
      writeBigInt64(encoder, data);
      break;
    case "object":
      if (data === null) {
        write(encoder, 126);
      } else if (Array.isArray(data)) {
        write(encoder, 117);
        writeVarUint(encoder, data.length);
        for (let i = 0; i < data.length; i++)
          writeAny(encoder, data[i]);
      } else if (data instanceof Uint8Array) {
        write(encoder, 116);
        writeVarUint8Array(encoder, data);
      } else {
        write(encoder, 118);
        const keys = Object.keys(data);
        writeVarUint(encoder, keys.length);
        for (let i = 0; i < keys.length; i++) {
          const key = keys[i];
          writeVarString(encoder, key);
          writeAny(encoder, data[key]);
        }
      }
      break;
    case "boolean":
      write(encoder, data ? 120 : 121);
      break;
    default:
      write(encoder, 127);
  }
}

export {
  Encoder,
  createEncoder,
  length,
  encodeToUint8Array,
  verifyLen,
  write,
  set,
  writeUint8,
  setUint8,
  writeUint16,
  setUint16,
  writeUint32,
  writeUint32BigEndian,
  setUint32,
  writeVarUint,
  isNegativeZero,
  writeVarInt,
  writeUint8Array,
  writeVarUint8Array,
  writeVarString,
  writeBinaryEncoder,
  writeOnDataView,
  writeFloat32,
  writeFloat64,
  writeBigInt64,
  writeBigUint64,
  writeAny
};
//# sourceMappingURL=chunk-MKG75UEC.js.map